# ###################################################################
# Import packages
# ###################################################################

import struct # for binary representation
import socket # for hostname identification
import string # for ascii selection
import numpy as np
from datetime import datetime, timedelta, timezone
from twisted.python import log

from martas.core import methods as mm
from martas.lib import publishing
from magpy.stream import read, DataStream, join_streams
import glob
import os


## MySQL protocol
## --------------------

class IMfileProtocol(object):
    """
    DESCRIPTION
       Protocol to read INTERMAGNET (actually MagPy) file data
       and broadcast/publish it in a MQTT stream.
       The selected data source (directory) is scanned for most
       recent data files matching a defined format.
       IMFile requires a path and a sensorid (if not contained in the file)
       IMFile is an active protocol, requesting data at defined periods.
       DataID name generation-is using sensorid and revision number (if provided)
    EXAMPLE
       sensors.cfg:
       WICadjusted,/home/leon/Cloud/Daten,-,-,-,-,active,None,30,1,IMfile,*.min,-,0001,-,-,-,magnetism,-

    """

    ## need a reference to our WS-MCU gateway factory to dispatch PubSub events
    ##
    def __init__(self, client, sensordict, confdict):
        self.client = client #self.wsMcuFactory = wsMcuFactory
        self.sensordict = sensordict
        self.confdict = confdict
        self.count = 0  ## counter for sending header information
        self.sensor = sensordict.get('sensorid')
        self.hostname = socket.gethostname()
        self.printable = set(string.printable)
        self.datalst = []
        self.datacnt = 0
        self.metacnt = 10
        self.revision = self.sensordict.get('revision','')
        self.payloadformat = confdict.get("payloadformat","martas")
        try:
            self.requestrate = int(self.sensordict.get('rate','-'))
        except:
            self.requestrate = 30

        if self.requestrate < 30:
            log.msg('  -> Requestrate below 30 not accepted for IMFile. Setting 30sec')
            self.requestrate = 30

        self.deltathreshold = confdict.get('timedelta')

        # debug mode
        debugtest = confdict.get('debug')
        self.debug = False
        if debugtest == 'True':
            log.msg('     DEBUG - {}: Debug mode activated.'.format(self.sensordict.get('protocol')))
            self.debug = True    # prints many test messages
        else:
            log.msg('  -> Debug mode = {}'.format(debugtest))

        # QOS
        self.qos=int(confdict.get('mqttqos',0))
        if not self.qos in [0,1,2]:
            self.qos = 0
        log.msg("  -> setting QOS:", self.qos)

        # IMFile specific
        self.lastt=None
        self.filewildcard = sensordict.get('name')
        self.pathname = sensordict.get('port')
        if not self.filewildcard:
            self.filewildcard = '*'
        log.msg("  -> checking for files:", self.pathname, self.filewildcard)
        filepath = self.get_latest_file(self.pathname,self.filewildcard)
        # get existing sensors for the relevant board
        if filepath:
            # check whether files as defined by the filename are existing at all
            log.msg("  -> SUCCESS: most recent file in given path-filename structure is {}".format(filepath))
            self.connectionMade(self.sensor)
        else:
            self.connectionLost(self.sensor,"Files corresponding to filestructure not found - check path")
            return

    def connectionMade(self, imfile):
        log.msg('  -> Filestructure {} connected.'.format(imfile))

    def connectionLost(self, imfile, reason=''):
        log.msg('  -> Filestructure access {} lost/not connectect. ({})'.format(imfile,reason))
        # implement counter and add three reconnection events here

    def get_latest_file(self,path,source):
        latest = []
        fullpath = os.path.join(path,source)
        if self.debug:
            log.msg("  -> DEBUG - checking for files: {}".format(fullpath))
        filelist = glob.glob(fullpath, recursive=True)
        filelist.sort(key=os.path.getctime)
        #TODO files = glob.glob('/ParentDirectory/**/*.csv', recursive=True)
        latest = filelist[-2:]
        return latest

    def get_headline_info(self,data):
        keys = data._get_key_headers()
        units, elements = [], []
        for key in keys:
            ele = data.header.get("col-{}".format(key),"")
            if not ele:
                ele = key
            un = data.header.get("unit-col-{}".format(key),"")
            if not un:
                un = "None"
            elements.append(ele)
            units.append(un)

        return keys, units, elements

    def sendRequest(self):
        """
        source:mysql:
        Method to obtain data from table
        """
        t1 = datetime.now(timezone.utc).replace(tzinfo=None)
        outdate = datetime.strftime(t1, "%Y-%m-%d")
        bufferfilename = outdate
        data = DataStream()

        if self.debug:
            log.msg("  -> DEBUG - Sending periodic request ...")

        # get source and identify latest filepath
        filepaths = self.get_latest_file(self.pathname,self.filewildcard)
        if self.revision:
            sensorid = self.sensor+"_"+self.revision
        else:
            sensorid = self.sensor
        if self.debug:
            log.msg("  -> DEBUG - dealing with sensor {}".format(sensorid))

        if self.debug:
            log.msg("  -> DEBUG - reading {}".format(filepaths))
        try:
            for filepath in filepaths:
                tmp = read(filepath)
                data = join_streams(data, tmp)
            if len(data) > 0:
                data = data.sorting()
            if self.debug:
                log.msg("  -> DEBUG - read {}, with {}".format(filepaths,len(data)))
        except:
            log.msg("  -> ERROR - accessing file at {}".format(filepath))

        if not len(data) > 0:
            if self.debug:
                log.msg("  -> DEBUG - obtained empty file structure")
            return

        # Trim data set and only work with most recent data dependend on request
        # get sampling rate
        ts,te = data.timerange()
        now = datetime.now(timezone.utc).replace(tzinfo=None)
        # always deal with a timerange twice of the request rate - why?
        tlow = now-timedelta(seconds=self.requestrate*2)
        #TODO define a suitable startcondition for tlow
        tlow = ts
        if not self.lastt:
            self.lastt=tlow

        if te<=tlow:
            if self.debug:
                log.msg("  -> DEBUG - no new data at {} - file covers until {}".format(now,te))
            return
        if self.debug:
            log.msg("  -> DEBUG - trimming data from {}".format(tlow))
        data = data.trim(starttime=self.lastt)
        self.lastt = te

        # obtain header information
        # required are lists from keys, elements and units
        keystab, units, elems = self.get_headline_info(data)
        if self.debug:
            log.msg("  -> DEBUG - creating head line with {}, {}, {}, {}".format(sensorid,keystab,units,elems))

        multplier = '['+','.join(map(str, [10000]*len(keystab)))+']'
        packcode = '6HL'+''.join(['q']*len(keystab))
        header = ("# MagPyBin {} {} {} {} {} {} {}".format(sensorid, '['+','.join(keystab)+']', '['+','.join(elems)+']', '['+','.join(units)+']', multplier, packcode, struct.calcsize('<'+packcode)))

        #transpose
        dataarray = data.ndarray
        orglist = [col for col in dataarray if len(col) == len(dataarray[0])]
        newli = np.transpose(orglist)

        for dataline in newli:
                timestamp = dataline[0].strftime("%Y-%m-%d %H:%M:%S.%f")
                data_bin = None
                datearray = ''
                try:
                    datearray = mm.time_to_array(timestamp)
                    for i,para in enumerate(keystab):
                        try:
                            val=int(float(dataline[i+1])*10000)
                        except:
                            val=999990000
                        datearray.append(val)
                    data_bin = struct.pack('<'+packcode,*datearray)  # little endian
                except:
                    log.msg('Error while packing binary data')

                if not self.confdict.get('bufferdirectory','') == '' and data_bin:
                    mm.data_to_file(self.confdict.get('bufferdirectory'), sensorid, bufferfilename, data_bin, header)
                if self.debug:
                    log.msg("  -> DEBUG - sending ... {}".format(','.join(list(map(str,datearray))), header))
                self.sendData(sensorid,','.join(list(map(str,datearray))),header,len(newli)-1, fullhead=data.header)

        if self.debug:
            t2 = datetime.now(timezone.utc).replace(tzinfo=None)
            log.msg("  -> DEBUG - Needed {}".format(t2-t1))


    def sendData(self, sensorid, data, head, stack=None, fullhead=None):
        if not fullhead:
            fullhead = {}

        topic = self.confdict.get('station') + '/' + sensorid
        senddata = False
        if not stack:
            stack = int(self.sensordict.get('stack'))
        coll = stack

        if coll > 1:
            self.metacnt = 1 # send meta data with every block
            if self.datacnt < coll:
                self.datalst.append(data)
                self.datacnt += 1
            else:
                senddata = True
                data = ';'.join(self.datalst)
                self.datalst = []
                self.datacnt = 0
        else:
            senddata = True

        if senddata:
                if self.payloadformat in ["intermagnet","immqtt","bgsmqtt"]:
                    if self.debug:
                        print ("Publishing INTERMAGNET", topic, data, head)
                    pubdict = publishing.intermagnet(None, topic=topic, data=data, head=head,
                                            imo=self.confdict.get('station', ''), meta=fullhead)
                    if self.debug:
                        print("Pubdict ", pubdict)
                else:
                    pubdict, count = publishing.martas(None, topic=topic, data=data, head=head, count=self.count,
                                            changecount=self.metacnt,
                                            imo=self.confdict.get('station', ''), meta=self.sensordict)
                    self.count = count
                for topic in pubdict:
                    if self.debug:
                        print ("Publishing", topic, pubdict.get(topic))
                    self.client.publish(topic, pubdict.get(topic), qos=self.qos)



#!/usr/bin/env python
# coding=utf-8

"""
MagPy - Backup and recover

martas_backup -p backup_path  - will create a zipped backup file containing configuration files, crontab inputs,
including info on python version, environment etc

martas_backup -r -p backup_path (or --recover)   -  will recover (plus init, if necessary) a martas stations after
pip install ... will issue warnining if python versions etc is different

TODO
add fstab and ntp.conf
"""

# Define packages to be used (local refers to test environment)
# ------------------------------------------------------------
import shutil
import os
from datetime import datetime

import getopt
from crontab import CronTab
import sys
import platform
import socket
import pathlib

from martas.version import __version__
from martas.core import methods as mm


def backup_files(source_dir, backup_dir):
    """
    DESCRIPTION
        backup configuration files
    """
    # Get the current date and time for naming the backup folder
    current_time = datetime.now().strftime('%Y-%m-%d')
    host = socket.gethostname()

    backup_folder = os.path.join(backup_dir, f"backup_{host}_{current_time}", ".martas")
    try:
        # Create the backup directory
        if not os.path.exists(backup_folder):
            os.makedirs(backup_folder)
        # Copy all files and subdirectories from source to backup folder
        if os.path.exists(backup_folder):
            shutil.rmtree(backup_folder)
        shutil.copytree(source_dir, backup_folder)
        print(f"Backup: Files have been copied to {backup_folder}")
    except Exception as e:
        print(f"Error: {e}")
    return os.path.join(backup_dir, f"backup_{host}_{current_time}")

def save_crontab(backup_folder, cronname="usercron.bak"):
    cronfile = os.path.join(backup_folder, cronname)
    with CronTab(user=True) as cron:
        cron.write(cronfile)

def create_info(backup_folder, martaspath):
    infofile = os.path.join(backup_folder, "README.txt")
    l = []
    l.append("# MARTAS BACKUP - SYSTEM INFORMATION")
    l.append("")
    l.append("MARTAS path: {}".format(martaspath))
    # add python version and environment here
    l.append("Python version: {}".format(sys.version))
    # add info on MARTAS or MARCOS, and main path
    l.append("MARTAS version: {}".format(__version__))
    # operating system
    l.append("Operating system: {} {}".format(platform.system(),platform.release()))
    with open(infofile, 'w') as f:
        for line in l:
            f.write(f"{line}\n")

def read_info(backup_folder):
    infofile = os.path.join(backup_folder, "README.txt")
    martaspath = ""
    with open(infofile, 'r') as f:
        for line in f:
            if line.startswith("MARTAS path: "):
                martaspath = line.replace("MARTAS path: ","").strip()
                print (" Will recover MARTAS directory: ~{} ".format(martaspath))
            if line.startswith("Python version: "):
                bckvers = line.replace("Python version: ","").strip()
                newvers = sys.version
                if not bckvers == newvers:
                    print(" Backup was running on Python {} ".format(bckvers))
                    print(" New system is running Python {} ".format(newvers))
            if line.startswith("MARTAS version: "):
                bckvers = line.replace("MARTAS version: ","").strip()
                newvers = __version__
                if not bckvers == newvers:
                    print(" Backup was running on MARTAS {} ".format(bckvers))
                    print(" New system is running MARTAS {} ".format(newvers))
            if line.startswith("Operating system: "):
                bckvers = line.replace("Operating system: ","").strip()
                newvers = "{} {}".format(platform.system(),platform.release())
                if not bckvers == newvers:
                    print(" Backup was running on {} ".format(bckvers))
                    print(" New system is running {} ".format(newvers))

    return martaspath.lstrip("/")


def recover_populate(homedir, debug=False):
    # recover main folder and crontab
    unpack = '/tmp/recover'
    if debug:
        print (" Debug mode activated - will not update anything")
    writecred = True
    if os.path.isfile(os.path.join(homedir, ".magpycred")):
        writecred = False
        print (" - credentials already existing!")
        print("   CHECK CAREFULLY - ALL EXISTING CONTENTS WILL BE ERASED")
        answer = input("  Replace existing credentials with backup? (y/n) ")
        if answer in ["yes", "y", "Y"] and not debug:
            writecred = True
    if writecred and not debug:
        shutil.copyfile(os.path.join(unpack, ".magpycred"), os.path.join(homedir, ".magpycred"))
    cronbck = os.path.join(unpack, "usercron.bak")
    file_cron = CronTab(tabfile=cronbck)
    if not debug:
        file_cron.write()
    # Remove README, usercron and .magpycred
    cron_to_rem = pathlib.Path(os.path.join(unpack, "usercron.bak"))
    read_to_rem = pathlib.Path(os.path.join(unpack, "README.txt"))
    cred_to_rem = pathlib.Path(os.path.join(unpack, ".magpycred"))
    cron_to_rem.unlink()
    read_to_rem.unlink()
    cred_to_rem.unlink()
    #if os.path.exists(os.path.join(homedir, ".magpycred")):
    bckfolders = next(os.walk(unpack))[1]
    for bckfolder in bckfolders:
        writefolder = True
        dest = os.path.join(homedir, bckfolder)
        source = os.path.join(unpack, bckfolder)
        if os.path.exists(dest):
            writefolder = False
            print (" - MARTAS main folder already existing!")
            print ("   CHECK CAREFULLY - ALL EXISTING CONTENTS WILL BE ERASED")
            answer = input("  Erase existing MARTAS folder and replace with backup? (y/n) ")
            if answer in ["yes","y", "Y"] and not debug:
                writefolder = True
                shutil.rmtree(dest)
        if writefolder and not debug:
            shutil.copytree(source, dest)
    return True



def main(argv):
    version = __version__
    homedir = os.getenv("HOME")
    backuppath = os.path.join(homedir,".martas")
    destpath = "/tmp"
    recoverpath = ''
    debug = False

    try:
        opts, args = getopt.getopt(argv, "hb:d:r:D",
                                   ["backup=", "destination=", "recover=", "debug="])
    except getopt.GetoptError:
        print('martas_backup.py -b <backup> -d <destination> -r <recover>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('-------------------------------------')
            print('Description:')
            print('-- martas_backup.py will backup a MARTAS/MARCOS machine --')
            print('-----------------------------------------------------------------')
            print('detailed description:')
            print(' martas_backup.py creates a backup of configurations and setup.')
            print(' Creating a backup of MARTAS files and configuartions ')
            print(' ... ')
            print(' Recover from backup:')
            print(' ...')
            print('')
            print('-------------------------------------')
            print('Usage:')
            print('python martas_backup.py -c <config>')
            print('-------------------------------------')
            print('Options:')
            print('-b            : backup - path to main dir, i.e. /home/user/.martas')
            print('-d            : destination to write backup to i.e. /home/user/backups')
            print("-r            : recover from backup")
            print('-------------------------------------')
            print('Application:')
            print('  Backup:  python3 martas_backup.py -b /home/user/.martas -d /home/user/backups')
            print('  Recover: python3 martas_backup.py -r /tmp/backup_theia_2025-06-21.zip')
            sys.exit()
        elif opt in ("-b", "--backup"):
            # delete any / at the end of the string
            backuppath = os.path.abspath(arg)
        elif opt in ("-d", "--destination"):
            # delete any / at the end of the string
            destpath = os.path.abspath(arg)
        elif opt in ("-r", "--recover"):
            recoverpath = os.path.abspath(arg)
        elif opt in ("-D", "--debug"):
            # delete any / at the end of the string
            debug = True

    print("Running martas backup version {}".format(version))
    print("--------------------------------")
    if not os.path.exists(destpath):
        os.makedirs(destpath)

    if backuppath and not recoverpath: # default as backuppath is tmp
        print ("Creating backup")
        if not os.path.exists(backuppath):
            print('!! Specify a valid path to the main MARTAS/MARCOS folder !!')
            sys.exit()
        # create a backup folder in tmp
        # copy main directory into the backup folder
        bpath = backup_files(backuppath, destpath)
        # save credential information in folder
        if os.path.isfile(os.path.join(homedir,".magpycred")):
            shutil.copyfile(os.path.join(homedir,".magpycred"), os.path.join(bpath,".magpycred"))
        # save crontab to folder
        save_crontab(bpath)
        # save pythonpath/info in backup readme
        martaspath = backuppath.replace(homedir,"")
        create_info(bpath, martaspath)
        # zip it and move to destination folder
        destination = os.path.join(destpath,os.path.basename(bpath))
        shutil.make_archive(destination, 'zip', bpath)
        shutil.rmtree(bpath)
        print(" backup saved to {}.zip".format(destination))
    elif recoverpath:
        print ("Recovering from backup")
        if not os.path.exists(recoverpath):
            print('!! Specify a valid path to a existing backup !!')
            sys.exit()
        # unpack zip
        shutil.unpack_archive(recoverpath, '/tmp/recover')
        # extract martaspath, pythonpath and info on main MARTAS/MARCOS folder
        # check validity and eventually send warning
        martaspath = read_info('/tmp/recover')
        # extract main folder, crontab and credentials
        recover_populate(homedir, debug=True)
    else:
        print ("No option selected")

    print("martas_backup successfully finished")


if __name__ == "__main__":
    main(sys.argv[1:])





# ###################################################################
# Import packages
# ###################################################################

import struct # for binary representation
import socket # for hostname identification
import string # for ascii selection
import sys
import numpy as np
from datetime import datetime, timezone
from twisted.protocols.basic import LineReceiver
from twisted.python import log
from martas.core import methods as mm


## GEM -GSM90 protocol
##
class GSM90Protocol(LineReceiver):
    """
    Protocol to read GSM90 data
    This protocol defines the individual sensor related read process.
    It is used to dipatch url links containing specific data.
    Sensor specific coding is contained in method "processData".

    Treatment of GPS and NTP time:
    The protocol determines two times: 1) GPS time provided by the data logger
    2) NTP time of the PC when data is received by serial comm.
    Primary time is selected in sensor.cfg (either NTP or GPS)
    Timediff between GPS and NTP is recorded, and a 1000 line median average is send
    within the DataDictionary (DataNTPTimeDelay).
    Besides, a waring message is generated if NTP and GPS times differ by more than 3 seconds.
    For this comparison (and only here) a typical correction (delay) value is applied to NTP.
    e.g. 6.2 sec for POS1.
    """
    def __init__(self, client, sensordict, confdict):
        """
        'client' could be used to switch between different publishing protocols
                 (e.g. MQTT or WS-MCU gateway factory) to publish events
        'sensordict' contains a dictionary with all sensor relevant data (sensors.cfg)
        'confdict' contains a dictionary with general configuration parameters (martas.cfg)
        """
        self.client = client
        self.sensordict = sensordict
        self.confdict = confdict
        self.count = 0  ## counter for sending header information
        self.sensor = sensordict.get('sensorid')
        self.hostname = socket.gethostname()
        self.printable = set(string.printable)
        #log.msg("  -> Sensor: {}".format(self.sensor))
        self.datalst = []
        self.datacnt = 0
        self.metacnt = 10
        self.errorcnt = {'time':0}

        self.delaylist = []  # delaylist contains up to 1000 diffs between gps and ntp
                             # the median of this values is used for ntp timedelay
        self.timedelay = 0.0
        self.timethreshold = 3 # secs - waring if timedifference is larger the 3 seconds

        # QOS
        self.qos=int(confdict.get('mqttqos',0))
        if not self.qos in [0,1,2]:
            self.qos = 0
        log.msg("  -> setting QOS:", self.qos)
        # PYTHON version
        self.pvers = sys.version_info[0]

    def connectionMade(self):
        log.msg('  -> {} connected.'.format(self.sensor))

    def connectionLost(self, reason):
        log.msg('  -> {} lost.'.format(self.sensor))

    def processData(self, data):
        """ GSM90 data """
        currenttime = datetime.now(timezone.utc).replace(tzinfo=None)
        outdate = datetime.strftime(currenttime, "%Y-%m-%d")
        actualtime = datetime.strftime(currenttime, "%Y-%m-%dT%H:%M:%S.%f")
        outtime = datetime.strftime(currenttime, "%H:%M:%S")
        timestamp = datetime.strftime(currenttime, "%Y-%m-%d %H:%M:%S.%f")
        filename = outdate
        sensorid = self.sensor
        packcode = '6hLLL6hL'
        header = "# MagPyBin %s %s %s %s %s %s %d" % (self.sensor, '[f,var1,sectime]', '[f,errorcode,internaltime]', '[nT,none,none]', '[1000,1,1]', packcode, struct.calcsize('<'+packcode))

        try:
            # Extract data
            # old data looks like 04-22-2015 142244  48464.53 99
            data_array = data
            if len(data) == 4:
                intensity = float(data[2])
                err_code = int(data[3])
                try:
                    try:
                        internal_t = datetime.strptime(data[0]+'T'+data[1], "%m-%d-%YT%H%M%S.%f")
                    except:
                        internal_t = datetime.strptime(data[0]+'T'+data[1], "%m-%d-%YT%H%M%S")
                    internal_time = datetime.strftime(internal_t, "%Y-%m-%d %H:%M:%S.%f")
                except:
                    internal_time = timestamp #datetime.strftime(datetime.now(timezone.utc).replace(tzinfo=None), "%Y-%m-%d %H:%M:%S.%f")
                #print internal_time
            elif len(data) == 3: # GSM v7.0
                intensity = float(data[1])
                err_code = int(data[2])
                try:
                    internal_t = datetime.strptime(outdate+'T'+data[0], "%Y-%m-%dT%H%M%S.%f")
                    internal_time = datetime.strftime(internal_t, "%Y-%m-%d %H:%M:%S.%f")
                except:
                    internal_time = timestamp #datetime.strftime(datetime.now(timezone.utc).replace(tzinfo=None), "%Y-%m-%d %H:%M:%S.%f")
            else:
                err_code = 0
                intensity = float(data[0])
                internal_time = timestamp #datetime.strftime(datetime.now(timezone.utc).replace(tzinfo=None), "%Y-%m-%d %H:%M:%S")
        except:
            log.err('{} protocol: Data formatting error. Data looks like: {}'.format(self.sensordict.get('protocol'),data))

        try:
            # Analyze time difference between GSM internal time and utc from PC
            timelist = sorted([internal_t,currenttime])
            timediff = timelist[1]-timelist[0]
            #secdiff = timediff.seconds + timediff.microseconds/1E6
            #timethreshold = 3
            delta = timediff.total_seconds()
            if not delta in [0.0, np.nan, None]:
                self.delaylist.append(timediff.total_seconds())
                self.delaylist = self.delaylist[-1000:]
            if len(self.delaylist) > 100:
                try:
                    self.timedelay = np.median(np.asarray(self.delaylist))
                except:
                    self.timedelay = 0.0
            #if secdiff > timethreshold:
            if delta > self.timethreshold:
                self.errorcnt['time'] +=1
                if self.errorcnt.get('time') < 2:
                    log.msg("{} protocol: large time difference observed for {}: {} sec".format(self.sensordict.get('protocol'), sensorid, delta))
            else:
                self.errorcnt['time'] = 0
        except:
            pass

        if self.sensordict.get('ptime','') in ['NTP','ntp']:
            secondtime = internal_time
            maintime = timestamp
        else:
            maintime = internal_time
            secondtime = timestamp

        try:
            ## GSM90 does not provide any info on whether the GPS reading is OK or not

            # extract time data
            datearray = mm.time_to_array(maintime)
            try:
                datearray.append(int(intensity*1000.))
                datearray.append(err_code)
                #print timestamp, internal_time
                internalarray = mm.time_to_array(secondtime)
                datearray.extend(internalarray)
                data_bin = struct.pack('<'+packcode,*datearray)
            except:
                log.msg('{} protocol: Error while packing binary data'.format(self.sensordict.get('protocol')))

            if not self.confdict.get('bufferdirectory','') == '':
                mm.data_to_file(self.confdict.get('bufferdirectory'), sensorid, filename, data_bin, header)

        except:
            log.msg('{} protocol: Error with binary save routine'.format(self.sensordict.get('protocol')))


        return ','.join(list(map(str,datearray))), header


    def lineReceived(self, line):

        topic = self.confdict.get('station') + '/' + self.sensordict.get('sensorid')
        # check python version and eventually decode binary string
        if self.pvers > 2:
            line=line.decode('ascii')
        # extract only ascii characters
        line = ''.join(filter(lambda x: x in string.printable, str(line)))

        ok = True
        try:
            data = line.split()
            data, head = self.processData(data)
        except:
            print('{}: Data seems not be GSM90Data: Looks like {}'.format(self.sensordict.get('protocol'),line))
            ok = False

        if ok:
            senddata = False
            coll = int(self.sensordict.get('stack'))
            if coll > 1:
                self.metacnt = 1 # send meta data with every block
                if self.datacnt < coll:
                    self.datalst.append(data)
                    self.datacnt += 1
                else:
                    senddata = True
                    data = ';'.join(self.datalst)
                    self.datalst = []
                    self.datacnt = 0
            else:
                senddata = True

            if senddata:
                self.client.publish(topic+"/data", data, qos=self.qos)
                if self.count == 0:
                    add = "SensorID:{},StationID:{},DataPier:{},SensorModule:{},SensorGroup:{},SensorDecription:{},DataTimeProtocol:{}".format( self.sensordict.get('sensorid',''),self.confdict.get('station',''),self.sensordict.get('pierid',''),self.sensordict.get('protocol',''),self.sensordict.get('sensorgroup',''),self.sensordict.get('sensordesc',''),self.sensordict.get('ptime','') )
                    self.client.publish(topic+"/dict", add, qos=self.qos)
                    self.client.publish(topic+"/meta", head, qos=self.qos)
                self.count += 1
                if self.count >= self.metacnt:
                    self.count = 0



#!/usr/bin/env python
# coding=utf-8

"""
Monitoring script:

allows for:
1) to check actuality for last buffer file
2) check actuality of data base tables
3) check disk space and memory
4) check database size
5) check LogFile for frequent messages
6) trigger the execution of external scripts if one of 1 to 4 happens

This script should be running on each martas machine.
It will produce a mqtt json statusmessage if any critical 
information is changing like disk size or buffer files are not 
written any more.

This script can also check for log file changes. If logfile differences contain repeated similar messages
(too be defined in the config) then it is possible to trigger the execution of an external bash script.

################################

add crontag to regularly run monitor (root)
sudo crontab -e
5  *  *  *  *  /usr/bin/python3 /path/to/monitor.py -c /path/to/conf.cfg -n MARCOS -j marcos  > /dev/NULL 2&>1
"""
# Define packges to be used (local refers to test environment)
# ------------------------------------------------------------
import os, sys, getopt
import glob
from datetime import datetime, timezone
import paho.mqtt.client as mqtt
import json
import socket
from magpy.stream import DataStream
from magpy.core import database
from magpy.core import methods as mpmeth
import magpy.opt.cred as mpcred
import numpy as np
import filecmp, shutil
import subprocess

from martas.core.methods import martaslog as ml
from martas.core import methods as mm
from martas.version import __version__

"""
monitorconf = {'logpath' : '/var/log/magpy/mm-monitor.log',		# path to log file
               'basedirectory' : '/srv', 			# base directory of buffer (MARTAS) and archive (MARCOS)
               'dbcredentials' : 'cobsdb', 			# where to find database credentials
               'defaultthreshold' : '600',  			# accepted age of data in file or database (in seconds)
               'ignorelist' : ['BASELINE','QUAKES','IPS','PIERS','DATAINFO','SENSORS','STATIONS','DIDATA_WIC','FLAGS'],  # sensors not too be checked
               'thresholds' : {'RCS':180000,'TILT':100000,'METEO':10800,'WIC':20000,'GAMMA':10800,'GWR':10800, 'LEMI036_3':180000, 'GSM90_6107632':180000, 'BMP085_10085004':180000, 'SHT75_RASHT004':180000, 'GSM90_7':180000, 'GP20S3EWstatus': 180000}, 		# threshold definitions
               'tmpdir' : '/tmp',			 	# for log file to check
               'logfile' : '/var/log/magpy/marcos.log', 	# log file to check
               'logtesttype' : 'repeat', 			# checks on log file: NEW (new input), REPEATed, LAST message (if a certain message is repeated more than x times)
               'logsearchmessage' : 'writeDB: unknown MySQL error when checking for existing tables!',
               'tolerance'  :  20,  				# tolerated amount of repeated messages
               'joblist' : ['space','martas','marcos','logfile'], 			# basic job list (can be space (only disk space), martas (buffer files), marcos (tables), logfile (logfiles)
               'execute'  :  '/path/execute.sh',  		# bash script to be executed if critical error is found (message contains 'execute'), add execution date to log
               'executecriteria'  :  'alternating',  	        # day (every day), week (once per week), alternating (try immidiatly, then one day later, finally one week later),
               'notification'  :  'telegram',  	        	# none,mail,telegram
               'notificationconf' : '/etc/martas/telegram.cfg', # configuration for notification
               'level'   :   3 }
"""

def _latestfile(path, date=False, latest=True):
    """
    DESCRIPTION
        get latest file
    """
    list_of_files = glob.glob(path) # * means all if need specific format then *.csv
    if len(list_of_files) > 0:
        if latest:
            latest_file = max(list_of_files, key=os.path.getctime)
        else:
            latest_file = min(list_of_files, key=os.path.getctime)
        ctime = os.path.getctime(latest_file)
        if date:
            return datetime.fromtimestamp(ctime)
        else:
            return latest_file
    else:
        return ""

def getspace(path,warning=80,critical=90): # path = '/srv'
    """
    DESCRIPTION
        get space
    """
    statvfs = os.statvfs(path)
    total = (statvfs.f_frsize * statvfs.f_blocks / (1024.*1024.))     # Size of filesystem in bytes
    remain = (statvfs.f_frsize * statvfs.f_bavail / (1024.*1024.))     # Number of free bytes that ordinary users
    usedper=100-(remain/total*100.)
    print ("Disk containing {} currently uses: {}%".format(path,usedper))
    #mesg = "status:\nDisk-size: {:.0f}MB\nDisk available: {:.0f}MB\nDisk occupied: {:.1f}%".format(total,remain, 100-(remain/total*100.))
    level = 'OK'
    if usedper >= warning:
        level = "warning: used > {}%".format(warning)
    if usedper >= critical:
        level = "critical: used > {}%".format(critical)
    return level


def _testJobs(joblist, allowedjobs):
    joblist = list(set([job for job in joblist if job in allowedjobs]))
    if len(joblist) == 0:
        print ("No valid jobs found")
        joblist = None
    return joblist


def check_martas(testpath='/srv', threshold=600, jobname='JOB', statusdict=None, ignorelist=None, thresholddict=None, debug=False):
    """
    DESCRIPTION:
        Walk through all subdirs of /srv and check for latest files in all subdirs
        add active or inactive to a log file
        if log file not exists: just add data
        if existis: check for changes and create message with all changes
    """
    if not statusdict:
        statusdict = {}
    if not ignorelist:
        ignorelist = []
    if not thresholddict:
        thresholddict = {}
    defaultthreshold = threshold
    # neglect archive, products and projects directories of MARCOS
    dirs=[x[0] for x in os.walk(testpath) if not x[0].find("archive")>-1 and not x[0].find("products")>-1 and not x[0].find("projects")>-1]
    for d in dirs:
        ld = _latestfile(os.path.join(d,'*'),date=True)
        lf = _latestfile(os.path.join(d,'*'))
        if os.path.isfile(lf):
            if debug:
                print ("Ckecking {} ...".format(lf))
            # check white and blacklists
            performtest = False
            if not any([lf.find(ig) > -1 for ig in ignorelist]):
                if debug:
                   print ("  not in ignorelist")
                now = datetime.now(timezone.utc).replace(tzinfo=None)
                diff = (now-ld).total_seconds()
                dname = d.split('/')[-1]
                state = "active"
                if any([lf.find(th) > -1 for th in thresholddict]):
                    matches = list(set([th for th in thresholddict if lf.find(th) > -1]))
                    try:
                        thresholds = [float(thresholddict.get(match)) for match in matches]
                        threshold = max(thresholds)
                        if debug:
                            print ("  using defined threshold of {}".format(threshold))
                    except:
                        pass
                if diff > threshold:
                    state = "inactive"
                threshold = defaultthreshold
                msgname = "{}-{}".format(jobname,dname.replace('_',''))
                statusdict[msgname] = state
                if debug:
                    print ("{}: {}".format(dname,state))
    return statusdict


def check_datafile(testpath='/srv/products/raw', threshold=600, jobname='JOB', statusdict=None, ignorelist=None, thresholddict=None, debug=False):
    """
    DESCRIPTION:
        Git to the directory testpath and check for latest files
        add active or inactive to a log file
        if log file not exists: just add data
        if exists: check for changes and create message with all changes
    """
    if not statusdict:
        statusdict = {}
    if not ignorelist:
        ignorelist = []
    if not thresholddict:
        thresholddict = {}

    ld = _latestfile(testpath,date=True)
    lf = _latestfile(testpath)
    if os.path.isfile(lf):
        if debug:
            print (" Latest file: {} ...".format(lf))

        # check white and blacklists
        performtest = False
        if not any([lf.find(ig) > -1 for ig in ignorelist]):
            if debug:
                print ("   -> not containd in ignorelist - continuing")
            now = datetime.now(timezone.utc).replace(tzinfo=None)
            diff = (now-ld).total_seconds()
            dname = testpath.split('/')[-1]
            state = "recent file found"
            if diff > threshold:
                state = "no recent file"
            msgname = "{}-{}".format(jobname,dname.replace('_','').replace('*',''))
            statusdict[msgname] = state
            if debug:
                print ("{}: {}".format(dname,state))
        else:
            print ("    -> containd in ignorelist - passing")

    return statusdict


def check_marcos(db,threshold=600, statusdict=None,jobname='JOB',excludelist=None,acceptedoffsets=None,debug=False):
    """
    DESCRIPTION
        add text
    """
    if not statusdict:
        statusdict = {}
    if not excludelist:
        excludelist = []
    if not acceptedoffsets:
        acceptedoffsets = {}

    testst = DataStream()
    offset = {}
    testname = '{}-DBactuality'.format(jobname)
    tables = []
    lasttime = None
    cursor = db.db.cursor()
    ok = True

    if debug:
        print ("1. Get all tables")
        print ("-----------------------------------")
    tablessql = 'SHOW TABLES'
    message = db._executesql(cursor,tablessql)
    if message:
        ok = False
        print (message)
    if ok:
        tables = cursor.fetchall()
        tables = [el[0] for el in tables]
        if not len(tables) > 0:
            print ('check table: no tables found - stopping')
            ok = False
    else:
        print ('check table: aborting')
        #cursor.close()
        ok = False

    if ok:
        if debug:
            print ("2. Extract tables to be examined")
            print ("-----------------------------------")
        if debug:
            print ("Data to be excluded: {}".format(excludelist))
        newtables = []
        for el in tables:
            drop = False
            for ex in excludelist:
                if el.startswith(ex):
                    drop = True
            if not drop:
                newtables.append(el)
        tables = newtables
        if debug:
            print ("Remaining tables: {}".format(tables))

    if ok:
        if debug:
            print ("3. Delete any existing timestamps which are in the future")
            # classic problem of IWT
            print ("-----------------------------------")
        delsql = "DELETE FROM IWT_TILT01_0001_0001 WHERE time > NOW()"
        if ok:
            message = db._executesql(cursor, delsql)
            if message:
                print(message)
        # eventually an execute is necessary here

    if ok:
        if debug:
            print ("4. Getting last input in each table")
            print ("-----------------------------------")
        for table in tables:
            if debug:
                print (' -> running for {}'.format(table))
            lastsql = 'SELECT time FROM {} ORDER BY time DESC LIMIT 1'.format(table)
            message = db._executesql(cursor, lastsql)
            if message:
                print(message)
            value = cursor.fetchall()
            try:
                lasttime = value[0][0]
                timetest = True
            except:
                timetest = False
                pass
            if timetest:
                lastt = mpmeth.testtime(lasttime)
                # Get difference to current time
                now = datetime.now(timezone.utc).replace(tzinfo=None)
                tdiff = np.abs((now-lastt).total_seconds())
                offset[table] = tdiff
                if debug:
                    print ("Difference: {}".format(tdiff))

    if ok:
        if debug:
            print ("5. Check threshold information")
            print ("-----------------------------------")
        statusdict[testname] = 'possible'
        for el in offset:
            # determine threshold
            usedthreshold = threshold
            name = "{}-{}".format(testname,el.replace('_',''))
            for elem in acceptedoffsets:
                if el.find(elem) > -1:
                    usedthreshold = acceptedoffsets[elem]
            if offset[el] > usedthreshold:
                if debug:
                    print ("{} : data too old by {} seconds".format(el,offset[el]))
                statusdict[name] = 'latest input older than {} sec'.format(usedthreshold)
            else:
                statusdict[name] = 'actual'
    else:
        statusdict[testname] = 'failure'
    cursor.close()

    return statusdict


def dbsize():
    # put that to a interactive command
    #    print ("1. Get initial database information")
    #    print ("-----------------------------------")
    #    dbinfo(db,destination='stdout',level='full')
    pass


def check_logfile(logfilepath, tmpdir='/tmp', statusdict=None, jobname='JOB', testtype='new', logsearchmessage='Error', tolerance=20, debug=False):
    """
    DESCRIPTION:
        read a log file and compare it with a copy in tmp (created in the last call)
    """
    # 1 Read log file
    # 2 Read copy in tmp
    # 3 Compare both files
    # or
    # 4 check for specific messages and occurrences (if not present in last 4 lines -> ignore)
    # 5 Save log to tmp
    if not statusdict:
        statusdict = {}

    def compare(f1,f2):
        diff = []
        with open(f1,'r') as f:
            d=f.readlines()
        with open(f2,'r') as f:
            e=f.readlines()
        lengthdiff = len(d)-len(e)
        if lengthdiff > 0:
            # new log is larger than old log
            # else do not return a difference (because of logrotate)
            diff = d[-lengthdiff:]
        return diff

    def last(f1, lines=None):
        with open(f1,'r') as f:
            d=f.readlines()
        if not lines:
            return d
        if lines and len(d) > lines:
            return d[-lines:]
        elif lines and len(d) < lines:
            return d
        else:
            return d

    testname = "{}-checklog".format(jobname)
    tmplogfilename = "monitor-{}.tmp".format(os.path.basename(logfilepath))
    tmplogfile = os.path.join(tmpdir,tmplogfilename)

    if not os.path.isfile(logfilepath):
        statusdict[testname] = "failed to find logfile"
        return statusdict

    if os.path.isfile(tmplogfile):
        # temporary file existing - running comparison
        res = filecmp.cmp(logfilepath, tmplogfile)
        checkname = "{}-content".format(testname)
        statusdict[checkname] = "log file ok"
        if res:
            if debug:
                print ("Log file did not change")
        else:
            if debug:
                print ("Log file changed")
            diff = compare(logfilepath, tmplogfile)
            if testtype == 'new' and len(diff) > 0:
                statusdict[checkname] = "new content: {}".format(diff[-3:])
            if testtype == 'repeat' and len(diff) > 0:
                # change of file is not important, only content in diff
                # analyse diff
                if len(diff) >= tolerance:
                    testamount = sum([el.find(logsearchmessage) > -1 for el in diff])
                    if testamount >= tolerance:
                        statusdict[checkname] = "CRITICAL: execute script"
        if testtype == 'last':
            # just check last line - independent from changes
            #  REQUIRES logsearchmessage to be success
            lines = last(logfilepath,2)
            if any([el.find(logsearchmessage) > -1 for el in lines]):
                if debug:
                    print ("Fine - found success message")
            else:
                statusdict[checkname] = "Did not find SUCCESS in {}".format(os.path.basename(logfilepath).replace("_",""))
        elif testtype == 'contain':
            # check all lines - independent from changes
            #  REQUIRES logsearchmessage to be success
            lines = last(logfilepath)
            if any([el.find(logsearchmessage) > -1 for el in lines]):
                if debug:
                    print ("Fine - found message {}".format(logsearchmessage))
            else:
                statusdict[checkname] = "Did not find {} in {}".format(logsearchmessage.replace("_",""), os.path.basename(logfilepath).replace("_",""))

    else:
        # Nothing to do ... create log file first
        pass

    # save logfile to tmp
    shutil.copy(logfilepath,tmpdir) # copy the logfile to destination dir
    tmplogtorename = os.path.join(tmpdir,os.path.basename(logfilepath))
    os.rename(tmplogtorename, tmplogfile) # rename

    return statusdict


def execute_script(call,statusdict=None,jobname='JOB',debug=True):
    """
    DESCRIPTION:
        find execute command in statusdict and execute the corresponding script
    """
    if not statusdict:
        statusdict = {}

    testname = "{}-execute".format(jobname)
    if debug:
        print ("Executing script ...")
    subprocess.call(call, shell=True)
    now = datetime.now(timezone.utc).replace(tzinfo=None)
    statusdict[testname] = now.strftime('%Y-%m-%d %H:%M')

    return statusdict

def main(argv):
    version = __version__
    statusmsg = {}
    jobs = ''
    joblist = []
    changes = []
    proxies = {}
    configpath = ''
    jobname = 'M{}'.format(version)
    hostname = socket.gethostname().upper()
    allowedjobs = ['martas','space','marcos','logfile','datafile']
    debug = False
    travistestrun = False
    #testst = DataStream()
    monitorconf = {'logpath' : '/var/log/magpy/mm-monitor.log',		# path to log file
               'basedirectory' : '/srv', 			# base directory of buffer (MARTAS) and archive (MARCOS)
               'dbcredentials' : 'cobsdb', 			# where to find database credentials
               'defaultthreshold' : '600',  			# accepted age of data in file or database (in seconds)
               'ignorelist' : ['BASELINE','QUAKES','IPS','PIERS','DATAINFO','SENSORS','STATIONS','DIDATA_WIC','FLAGS'],  # sensors not too be checked
               'thresholds' : {'RCS':180000,'TILT':100000,'METEO':10800,'WIC':20000,'GAMMA':10800,'GWR':10800, 'LEMI036_3':180000, 'GSM90_6107632':180000, 'BMP085_10085004':180000, 'SHT75_RASHT004':180000, 'GSM90_7':180000, 'GP20S3EWstatus': 180000}, 		# threshold definitions
               'tmpdir' : '/tmp',			 	# for log file to check
               'logfile' : '/var/log/magpy/marcos.log', 	# log file to check
               'logtesttype' : 'repeat', 			# checks on log file: new, contain, last, repeat
               'logsearchmessage' : 'writeDB: unknown MySQL error when checking for existing tables!',
               'tolerance'  :  20,  				# tolerated amount of repeated messages
               'joblist' : ['space','martas','marcos','logfile'], 			# basic job list (can be space (only disk space), martas (buffer files), marcos (tables), logfile (logfiles)
               'execute'  :  '/path/execute.sh',  		# bash script to be executed if critical error is found (message contains 'execute'), add execution date to log
               'executecriteria'  :  'alternating',  	        # day (every day), week (once per week), alternating (try immidiatly, then one day later, finally one week later),
               'notification'  :  'telegram',  	        	# none,mail,telegram
               'notificationconf' : '/etc/martas/telegram.cfg', # configuration for notification
               'level'   :   3 }


    try:
        opts, args = getopt.getopt(argv,"hc:n:j:vDT",["config=","jobname=","joblist=",])
    except getopt.GetoptError:
        print ('monitor.py -c <config> -n <jobname> -j <joblist> -v <version>')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print ('------------------------------------------------------------')
            print ('Description:')
            print ('-- monitor.py to monitor MARTAS/MARCOS machines and logs  --')
            print ('------------------------------------------------------------')
            print ('monitor.py is a python program for testing data actuality,')
            print ('get changes in log files, and get warnings if disk space is')
            print ('getting small.')
            print ('Therefore is is possible to monitor most essential aspects')
            print ('of data acquisition and storage. Besides, monitor.py can ')
            print ('be used to trigger external scripts in case of an observed')
            print ('"CRITICAL: execute script." state.')
            print ('monitor requires magpy >= 2.0.0.')
            print ('Jobs:')
            print ('space    : testing for disk size of basedirectory (i.e. /srv/mqtt or srv/archive)')
            print ('martas   : check for latest file updates in basedirectory and subdirs')
            print ('datafile : check for latest file updates only in basedirectory, not in subdirs')
            print ('marcos   : check for latest timestamp in data tables')
            print ('logfile  : log-test-types are: new or repeat, last, contains;')
            print ('         : new -> logfile has been changed since last run')
            print ('         : repeat -> checks for repeated logsearchmessage in changed logs')
            print ('         :           if more than tolerance then through execute script msg')
            print ('         : last -> checks for logsearchmessage in last two lines')
            print ('         : contains -> checks for logsearchmessage in full logfile')
            print ('-------------------------------------')
            print ('Usage:')
            print ('monitor.py -c <config> -n <jobname> -j <joblist>')
            print ('-------------------------------------')
            print ('Options:')
            print ('-c (required) : path to a configuration file')
            print ('-n            : give a name to the monitoring job')
            print ('-j            : override the joblist in conf')
            print ('-v            : print the current version of monitor.py')
            print ('-------------------------------------')
            print ('Application:')
            print ('python3 monitor.py -c /etc/martas/appconf/monitor.cfg')
            print ('python3 monitor.py -c /etc/martas/appconf/monitor.cfg -n DATABASE -j marcos')
            print ('python3 monitor.py -c /etc/martas/appconf/monitor.cfg -n MARTAS -j martas,space')
            print ('python3 monitor.py -c /etc/martas/appconf/monitor.cfg -n MARCOSLOG -j log')
            print ('python3 monitor.py -c /etc/martas/appconf/monitor.cfg -n AGEOFDATAFILE -j datafile')
            sys.exit()
        elif opt in ("-c", "--config"):
            configpath = arg
        elif opt in ("-n", "--jobname"):
            jobname = arg
        elif opt in ("-j", "--joblist"):
            jobs = arg
        elif opt == "-v":
            print ("monitor.py version: {}".format(version))
        elif opt in ("-D", "--debug"):
            debug = True
        elif opt in ("-T", "--test"):
            travistestrun = True

    # Testing inputs
    # --------------------------------
    if configpath == '':
        # create a config file in /etc/martas/appconf/monitor.cfg
        # use default monitorconf
        joblist = monitorconf.get('joblist')
        if not isinstance(joblist,list):
            joblist = [joblist]
        joblist = _testJobs(joblist,allowedjobs)
    else:
        if os.path.isfile(configpath):
            print (" reading configuration file ...")
            monitorconf = mm.get_conf(configpath)
            # directly get the joblist
            joblist = monitorconf.get('joblist')
            if not isinstance(joblist,list):
                joblist = [joblist]
            joblist = _testJobs(joblist,allowedjobs)
        else:
            print ('Specify a valid path to a configuration file')
            print ('-- check monitor.py -h for more options and requirements')
            sys.exit()
    if jobname == '':
        print ('An empty jobname is not allowed - using MARTASMONITOR')
        jobname = 'M'
    else:
        jobname = str(jobname)

    if jobs:
        tjoblist = jobs.split(',')
        tjoblist = _testJobs(tjoblist,allowedjobs)
        if tjoblist:
           joblist = tjoblist
    if not joblist:
        print ('Specify a valid job within the joblist of the conf file or dierctly')
        print ('-- check monitor.py -h for more options and requirements')
        sys.exit()

    # Preconfigure logging
    # --------------------------------
    testname = "{}-{}-monitor".format(hostname, jobname)
    logpath = monitorconf.get('logpath')
    if debug:
        print (testname, logpath)

    # Extract configuration data
    # --------------------------------
    receiver = monitorconf.get('notification')
    receiverconf = monitorconf.get('notificationconf')
    basedirectory = monitorconf.get('basedirectory')
    defaultthreshold = int(monitorconf.get('defaultthreshold',600))
    ignorelist = monitorconf.get('ignorelist')
    thresholddict = monitorconf.get('thresholds')
    dbcred = monitorconf.get('dbcredentials')
    tmpdir = monitorconf.get('tmpdir')
    logfile = monitorconf.get('logfile')
    logtesttype = monitorconf.get('logtesttype')
    testamount = int(monitorconf.get('tolerance'))
    logsearchmessage = monitorconf.get('logsearchmessage')
    execute = monitorconf.get('execute',None)
    spacewarning  = int(monitorconf.get('spacewarning', 80))
    spacecritical  = int(monitorconf.get('spacecritical', 90))

    if execute == "/path/execute.sh":
        execute = None

    if not isinstance(ignorelist, list):
        ignorelist = []
    if debug:
        print (receiver, receiverconf, basedirectory, defaultthreshold)
        print (ignorelist)

    # Run the main program
    # --------------------------------
    if debug:
        print ("Running the following jobs: {}".format(joblist))
    #spacename = "{}-{}-diskspace".format(hostname, jobname)
    #statusmsg[spacename] = getspace(basedirectory)
    try:
        if 'space' in joblist:
            if debug:
                print ("Running space job")
            try:
                spacename = "{}-{}-diskspace".format(hostname, jobname)
                if os.path.exists(basedirectory):
                    statusmsg[spacename] = getspace(basedirectory,warning=spacewarning,critical=spacecritical)
                else:
                    statusmsg[spacename] = "{} does not exist".format(basedirectory)
            except:
                statusmsg['diskspace'] = "Checking disk space failed"
        if 'martas' in joblist:
            if debug:
                print ("Running martas job")
            try:
                statusmsg = check_martas(testpath=basedirectory, threshold=defaultthreshold, jobname=testname, statusdict=statusmsg, ignorelist=ignorelist,thresholddict=thresholddict, debug=debug)
            except:
                statusmsg[testname] = "error when running martas job monitoring - please check"
        elif 'datafile' in joblist:
            if debug:
                print ("Running datafile job on {}".format(basedirectory))
            try:
                statusmsg = check_datafile(testpath=basedirectory, threshold=defaultthreshold, jobname=testname, statusdict=statusmsg, ignorelist=ignorelist,thresholddict=thresholddict, debug=debug)
            except:
                statusmsg[testname] = "error when running datafile monitoring - please check"
        if 'marcos' in joblist:
            if debug:
                print ("Running marcos job")
            try:
                db = mm.connect_db(dbcred)
                statusmsg = check_marcos(db, threshold=defaultthreshold, jobname=testname, statusdict=statusmsg, excludelist=ignorelist,acceptedoffsets=thresholddict, debug=debug)
            except:
                statusmsg[testname] = "error when running marcos job monitoring - please check"
        if 'logfile' in joblist:
            if debug:
                print ("Running logfile job on {}".format(logfile))
            try:
                statusmsg = check_logfile(logfile, tmpdir=tmpdir, jobname=testname, statusdict=statusmsg, testtype=logtesttype, logsearchmessage=logsearchmessage, tolerance=testamount, debug=debug)
            except:
                statusmsg[testname] = "error when running logfile monitoring - please check"
        if execute:
            # scan statusmessages for execute call
            try:
                if any([statusmsg.get(stat).find('CRITICAL: execute script')>-1 for stat in statusmsg]):
                    # Found a critical execution message
                    if debug:
                        print ("Running execute job")
                    statusmsg = execute_script(execute,jobname=testname, statusdict=statusmsg)
            except:
                statusmsg[testname] = "error when executing monitoring application"

        statusmsg[testname] = "monitoring application running successful"
    except:
        statusmsg[testname] = "basic error when running monitoring application - please check"

    if monitorconf.get('https'):
        proxies['https'] = monitorconf.get('https')
    if monitorconf.get('http'):
        proxies['http'] = monitorconf.get('http')

    if debug:
        print (statusmsg)
    else:
        martaslog = ml(logfile=logpath,receiver=receiver,proxies=proxies)
        if receiver == 'email':
            martaslog.email['config'] = receiverconf
        elif receiver == 'telegram':
            martaslog.telegram['config'] = receiverconf
        changes = martaslog.msg(statusmsg)

    print ("----------------------------------------------------------------")
    print ("monitoring app finished")
    print ("----------------------------------------------------------------")
    print ("SUCCESS")
    if travistestrun:
        return True

if __name__ == "__main__":
   main(sys.argv[1:])



"""
# EXAMPLE MONITOR CONFIGURATION FILE CONTENT:
# ###########################################

# path to log file
logpath   :   /var/log/magpy/mm-monitor.log

# base directory of buffer (MARTAS) and archive (MARCOS; or directory to check for file dates
basedirectory   :   /srv

# where to find database credentials
dbcredentials   :   cobsdb

# accepted age of data in file or database (in seconds)
defaultthreshold   :   600

# sensors not too be checked
ignorelist   :   BASELINE,QUAKES,IPS,PIERS,DATAINFO,SENSORS,STATIONS,DIDATA_WIC,FLAGS

# threshold definitions
#thresholds   :   {'RCS':180000,'TILT':100000,'METEO':10800,'WIC':20000,'GAMMA':10800,'GWR':10800, 'LEMI036_3':180000, 'GSM90_6107632':180000, 'BMP085_10085004':180000, 'SHT75_RASHT004':180000, 'GSM90_7':180000, 'GP20S3EWstatus': 180000}

# for log file to check
tmpdir   :   /tmp

# log file to check
logfile   :   /var/log/magpy/marcos.log

# checks on log file: NEW (new input), REPEATed message (if a certain message is repeated more than x times)
logtesttype   :   repeat

logsearchmessage   :   writeDB: unknown MySQL error when checking for existing tables!

# tolerated amount of repeated messages
tolerance   :   20

# basic job list (can be space (only disk space), martas (buffer files), marcos (tables), logfile (logfiles), datafile (age of file in directory)
joblist   :   space,martas,marcos,logfile

# bash script to be executed if critical error is found (message contains 'execute'), add execution date to log
execute   :   /path/execute.sh	

# day (every day), week (once per week), alternating (try immidiatly, then one day later, finally one week later)
executecriteria   :   alternating

# none,mail,telegram
notification   :   telegram

# configuration for notification
notificationconf   :   /etc/martas/telegram.cfg
"""



#!/usr/bin/env python
"""

DI.PY
Applictaion to analysze di data from different sources
######################################################

di.py can analyze data from various sources by considering data from
several different variometers and scalar systems. It further supports
data from multiple piers with different characteristics. Please note:
directional pier differences (dD and dI) are not considered.


How does it work:

1. - Needs access to all available variometers
   - Needs access to all available scalar sensors (first is primary)
   -> Primary access is database
   -> if not present or no data available there go to archive
   -> no success in archive: test for reported files on ftp accout
        (fallback in case the process is not running with servers access)
   => Test access and report any errors

   Parameters: -dbcred, id-list-vario, id-list-scalar, path-to-archive, link-to-fallback, fallback-cred

Get files from a remote server (to be reached by nfs, samba, ftp, html or local directory)
File content is directly added to a data bank (or local file if preferred).
"""
from __future__ import print_function

from magpy.stream import *
from magpy.absolutes import *
import magpy.mpplot as mp
from magpy.database import *
from magpy.opt import cred as mpcred

import getopt
import fnmatch
import pwd, grp  # for changing ownership of web files

scriptpath = os.path.dirname(os.path.realpath(__file__))
docdir = os.path.abspath(os.path.join(scriptpath, '..', 'doc'))
sys.path.insert(0, docdir)
from version import __version__

def walkdir(filepat,top):
    for path, dirlist, filelist in os.walk(top):
        for name in fnmatch.filter(filelist,filepat):
            yield os.path.join(path,name)


def main(argv):
    creddb = ''				# c
    dipath = ''				# a
    variolist = ''			# v
    variodataidlist = ''		# j
    scalarlist = ''			# s
    scalardataidlist = ''		# k
    pierlist = ''			# p
    abstypelist = ''			# y
    azimuthlist = ''			# z
    archive = ''			# w   (e.g. /srv/archive)
    identifier = 'BLV'                 # f
    stationid = 'wic'			# t
    fallbackvariopath = ''		# o
    fallbackscalarpath = ''		# l
    begin = '1900-01-01'		  		  # b
    end = datetime.strftime(datetime.utcnow(),"%Y-%m-%d") # e
    expD = 3				# d
    expI = 64				# i
    compensation=False                  # m
    rotation=False                      # q
    dbadd = False			# n
    addBLVdb = False			# n
    flagging=False			# g
    createarchive=False			# r
    webdir = '/var/www/joomla/images/didaten/' # TODO add option
    webuser = 'www-data'
    webgroup = 'www-data'
    defaultuser = 'cobs'
    defaultgroup = 'cobs'
    debug=False

    flaglist = []
    keepremote = False
    getremote = False
    remotecred = ''
    remotepath = ''
    variopath = ''			# 
    scalarpath = ''			# 

    try:
        opts, args = getopt.getopt(argv,"hc:a:v:j:s:k:o:mql:b:e:t:z:d:i:p:y:w:f:ngrx:u:D",["cred=","dipath=","variolist=","variodataidlist=","scalarlist=","scalardataidlist=","variopath=","compensation=","rotation=","scalarpath=","begin=","end=","stationid=","pierlist=","abstypelist=","azimuthlist=","expD=","expI=","write=","identifier=","add2DB=","flag=","createarchive=","webdir=","keepremote","debug="])
    except getopt.GetoptError:
        print('di.py -c <creddb> -a <dipath> -v <variolist>  -j <variodataidlist> -s <scalarlist> -o <variopath> -m <compensation> -q <rotation> -l <scalarpath> -b <startdate>  -e <enddate> -t <stationid>  -p <pierlist> -z <azimuthlist> -y <abstypelist> -d <expectedD> -i <expectedI> -w <writepath> -f<identifier> -n <add2DB>  -g  <flag> -r <createarchive> -x <webdir> -u <user> --keepremote')
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            print('-------------------------------------')
            print('Description:')
            print('dianalysis.py reads DI measurements and calculates DI values.')
            print('Provide variometer and scalar data for correction.')
            print('Returns di values, f and collimation angles.')
            print('A number of additional option allow for archiving, validity tests,')
            print('and output redirection. If variometer data is provided, base values')
            print('are calculated. ')
            print('')
            print('-------------------------------------')
            print('Usage:')
            print('di.py -c <creddb> -a <dipath> -v <variolist>  -j <variodataidlist> -s <scalarlist> -o <variopath> -l <scalarpath> -m <compensation> -q <rotation> -b <startdate>  -e <enddate> -t <stationid>  -p <pierlist> -z <azimuthlist> -y <abstypelist> -d <expectedD> -i <expectedI> -w <writepath> -n <add2DB>  -g  <flag> -r <createarchive> -x <webdir> -u <user> --keepremote')
            print('-------------------------------------')
            print('Options:')
            print('-c            : provide the shortcut to the data bank credentials')
            print('-a (required) : path to DI data - can be either a real path to a local directory')
            print('                    or a credential shortcut for a remote connection ')
            print('-w (required) : archivepath for writing results and eventually accessing data.')
            print('                e.g. /srv/archive. or /tmp') 
            print('                below this folder the following structure will be implemented:') 
            print('                -/srv/archive/"StationID"/DI/analyse : ')
            print('                                 folder with raw data to be analyzed') 
            print('                -/srv/archive/"StationID"/DI/data : ')
            print('                                 folder with calculated di results') 
            print('                                 Name will be set to "BLV_" + variometerID + pier.')
            print('                                 This plain text file can be opend and') 
            print('                                 analyzed with the MagPy stream package.') 
            print('                -/srv/archive/"StationID"/DI/raw : ')
            print('                             archivefolder with successfully analyzed raw data') 
            print('-f            : idetifier for BLV data (in database and filename) - default is BLV')
            print('-v            : variolist - comma separated list of variometer ids')
            print('-j            : variodataidlist - specify the dataids to be used fro each vario')
            print('                Default: 0002 for each variometer sensor')
            print('-o            : path to variometer data')
            print('-m (no input) : apply compensation field values to variometer data')
            print('-q (no input) : apply rotation to variometer data')
            print('-s            : scalarpath - path to scalar data')
            print('-k            : scalardataidlist - specify the dataids to be used fro each scalar')
            print('                Default: 0002 for each scalar sensor')
            print('-b            : startdate - begin of analysis  (not yet active)')
            print('-e            : enddate - default is today (not yet active)')
            print('-t (required) : ID of the station i.e. the Observatory code (required if')
            print('                not in meta data of DI measurements)')
            print('-z            : list of astronomic azimuths of the mark from the measurement pier')
            print('              : Azimuthlist needs either to be empty or to have the same order ')
            print('                and length of the pierlist.')
            print('                use "False" if the specific value should be taken from the')
            print('                originalfile/database')
            print('                e.g. -p D3,H6  -z False,156.678')
            print('-y            : comma separated list of absolute data types for each pier')
            print('              : "di" for standard theodolite or "autodif" ')
            print('              : e.g. -p D3,H6 -m di,autodif ')
            print('-d            : expected declination')
            print('-i            : expected inclination')
            print('-p (required) : name/number of the pier, comma separated list') 
            print('-n (no input) : add di and basevalues to data base')
            print('-g (no input) : read flaglist from DB if db is opened and add flags')
            print('-r            : move successfully analyzed files to raw archive')
            print('-x            : directory to copy non-analyzed files to.')
            print('              : can be a www directory at which PHP-scripts are used to edit data.')
            print('-u            : define user for which jobs are performed')
            print('              : e.g. cobs:cobsgroup')
            print('--keepremote  : Don t delete remote files after dowloading them')
            print('-------------------------------------')
            print('Examples:')
            print('1. Running on MARCOS servers:')
            print('python di.py -c wic -a cobshome,cobenzlabs -v "FGE_S0252_0001"')
            print('      -s "POS1_N432_0001" -j 0002 -b 2014-01-01')
            print('      -w /media/DAE2-4808/archive -d 3 -i 64 -t wic -p H1,A7,A2,A16')
            print('      -y di,di,di,autodif -z False,179.8978,180.1391,267.3982')
            print('2. Running it with manually provided data links:')
            print('python di.py -c wic -a /media/DAE2-4808/archive/WIC/DI/analyze')
            print('      -v "FGE_S0252_0001" -s "POS1_N432_0001" -j 0002 -b 2014-02-01')
            print('      -e 2014-05-01 -w /media/DAE2-4808/archive -d 3 -i 64 -t wic')
            print('      -p H1,A7,A2,A16 -y di,di,di,autodif -r')
            print('      -z False,179.8978,180.1391,267.3982 -u user:group')
            print('python di.py -c cobs -a cobshomepage,cobenzlabs ')
            print('      -v DIDD_3121331_0002,LEMI025_1_0002 -s DIDD_3121331_0002')
            print('      -j 0001,0001 -b 2014-10-01 -e 2014-10-07')
            print('      -w /srv/archive -d 3 -i 64 -t wik -p D -n -r')
            sys.exit()
        elif opt in ("-c", "--creddb"):
            creddb = arg
        elif opt in ("-a", "--dipath"):
            dipath = arg
        elif opt in ("-w", "--archive"):
            archive=arg
        elif opt in ("-f", "--identifier"):
            identifier=arg
        elif opt in ("-v", "--variolist"):
            variolist = arg.split(',')
        elif opt in ("-j", "--variodataidlist"):
            variodataidlist = arg.split(',')
        elif opt in ("-s", "--scalarlist"):
            scalarlist = arg.split(',')
        elif opt in ("-k", "--scalardataidlist"):
            scalardataidlist = arg.split(',')
        elif opt in ("-o", "--variopath"):
            fallbackvariopath = arg
        elif opt in ("-m", "--compensation"):
            compensation=True
        elif opt in ("-q", "--rotation"):
            rotation=True
        elif opt in ("-l", "--scalarpath"):
            fallbackscalarpath = arg
        elif opt in ("-b", "--begin"):
            begin = arg
        elif opt in ("-e", "--end"):
            end = arg
        elif opt in ("-t", "--stationid"):
            stationid = arg
        elif opt in ("-p", "--pierlist"):
            pierlist = arg.split(',')
        elif opt in ("-z", "--azimuthlist"):
            azimuthlist = arg.split(',')
        elif opt in ("-y", "--abstypelist"):
            abstypelist = arg.split(',')
        elif opt in ("-x", "--webdir"):
            webdir = arg
        elif opt in ("-u", "--user"):
            user = arg.split(':')
            if len(user) > 1:
                defaultuser = user[0]
                defaultgroup = user[1]
        elif opt in ("-d", "--expectedD"):
            try:
                expD = float(arg)
            except:
                print("expected declination needs to be a float")
                sys.exit()
        elif opt in ("-i", "--expectedI"):
            try:
                expI = float(arg)
            except:
                print("expected inclination needs to be a float")
                sys.exit()
        elif opt in ("-n", "--add2db"):
            dbadd = True
            addBLVdb = True
        elif opt in ("-g", "--flag"):
            flagging=True
        elif opt in ("--keepremote"):
            keepremote=True
        elif opt in ("-r", "--createarchive"):
           createarchive=True
        elif opt in ("-D", "--debug"):
           debug=True

    print ("-------------------------------------")
    print ("Starting di analysis ... MARTAS version {}".format(__version__))
    print ("-------------------------------------")

    if dipath == '':
        print(' Specify the path to the DI data: -a /path/to/my/data !')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()

    if archive == '':
        print(' Specify an Archive path for writing results: -w /path/to/my/archive !')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()

    if variolist == '':
        variolist = []
    if scalarlist == '':
        scalarlist = []
    if azimuthlist == '':
        azimuthlist = []
    if abstypelist == '' or len(abstypelist) == 0:
        abstypelist = ['di' for elem in pierlist]
    if variodataidlist == '':
        variodataidlist = []
    if scalardataidlist == '':
        scalardataidlist = []
    if len(variodataidlist) == 0:
        variodataidlist = ['0002' for elem in variolist]
    else:
        if not len(variolist) == len(variodataidlist):
            print(' You need to specify a specific DataID for each variometer: e.g. -j 0002,0001')
            print(' -- check dianalysis.py -h for more options and requirements')
            sys.exit()
    if len(scalardataidlist) == 0:
        scalardataidlist = ['0002' for elem in scalarlist]
    else:
        if not len(scalarlist) == len(scalardataidlist):
            print(' You need to specify a specific DataID for each variometer: e.g. -j 0002,0001')
            print(' -- check dianalysis.py -h for more options and requirements')
            sys.exit()


    if not len(abstypelist) == 0:
        if not len(abstypelist) == len(pierlist):
            print(' Abstypelist needs to have the same order and length of the pierlist')
            print(' -- check dianalysis.py -h for more options and requirements')
            sys.exit()

    try:
        test = datetime.strptime(begin,"%Y-%m-%d")
        print(test)
    except:
        print(' Date format for begin seems to be wrong: -b 2013-11-22')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()

    try:
        datetime.strptime(end,"%Y-%m-%d")
    except:
        print(' Date format for end seems to be wrong: -e 2013-11-22')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()

    if pierlist == []:
        print(' Specify a list of the measurement piers containing at list one element: -p [Pier2]')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()

    if not len(azimuthlist) == 0:
        if not len(azimuthlist) == len(pierlist):
            print(' Azimuthlist needs to have the same order and length of the pierlist')
            print(' -- check dianalysis.py -h for more options and requirements')
            sys.exit()

    if stationid == '':
        print(' Specify a station name e.g. your observatory code')
        print(' -- check dianalysis.py -h for more options and requirements')
        sys.exit()
    else:
        stationid = stationid.upper()

    if not creddb == '':
        print("  Accessing data bank ...")
        try:
            db = mysql.connect (host=mpcred.lc(creddb,'host'),user=mpcred.lc(creddb,'user'),passwd=mpcred.lc(creddb,'passwd'),db =mpcred.lc(creddb,'db'))
            print("  ... success")
        except:
            print("  ... failure - check your credentials")
            sys.exit()
    else:
        db = False

    if not fallbackvariopath == '':
        if not fallbackvariopath.endswith('*'):
            fallbackvariopath = os.path.join(fallbackvariopath,'*')
        variopath = fallbackvariopath

    if not fallbackscalarpath == '':
        if not fallbackscalarpath.endswith('*'):
            fallbackscalarpath = os.path.join(fallbackscalarpath,'*')
        scalarpath = fallbackscalarpath

    if variolist == []:
        if fallbackvariopath == '':
            print('  !! You have not provided any variometer information at all')

    if scalarlist == []:
        if fallbackscalarpath == '':
            print('  You have not provided any independent scalar information')
            print('  -> we asume this data is provided along with the DI files')

    # -----------------------------------------------------
    # a) Basic information
    # -----------------------------------------------------
    if debug:
        print (" -------------------------------------")
        print (" Archive", archive)
        print (" Variolist", variolist)
        print (" Abstypelist", abstypelist)
        print (" Dipath", dipath)

    # -----------------------------------------------------
    # b) Getting new raw data from the input server
    # -----------------------------------------------------
    print (" -------------------------------------")
    print (" Identifying DI data source")
    if not os.path.exists(dipath):
        print("  Checking given DI path for credential information ...")
        try:
            credlist = mpcred.sc()
            credshort = [elem[0] for elem in credlist]
            print("  ... found credentials")
        except:
            print(" dipath {} not existing - credentials not accessible - aborting".format(dipath))
            sys.exit()
        try:
            dic = dipath.split(',')
            print(dic)
            print(len(dic))
            if len(dic) == 2:
                remotecred = dic[0]
                remotepath = dic[1]
                print("  Using credentials {} to get DI data from the remote path {}".format(dic[0],dic[1]))
            elif len(dic) == 1:
                remotecred = dic[0]
                remotepath = ''
                print("  Using credentials {} to get DI data".format(dic[0]))
            else:
                print("  -> could not interprete dipath in terms of credential information")
                sys.exit()
            if remotecred in credshort:
                getremote = True
            else:
                print("  -> dipath %s not existing - credentials not existing - aborting" % dipath)
        except:
            print("  -> dipath %s not existing - credentials not existing - aborting" % dipath)
            sys.exit()
        if getremote == False:
            sys.exit()
    else:
        print("  Found directory at specified dipath location")

    # Getting data from the webdir (eventually edited and corrected)
    if createarchive and not webdir == '':
        print (" -------------------------------------")
        print (" Createarchive and webdir selected ...")
        dipath = os.path.join(archive,stationid,'DI','analyze')
        for pier in pierlist:
            diid = pier + '_' + stationid + '.txt'
            for infile in iglob(os.path.join(webdir,'*'+diid)):
                # Testing whether file exists:
                if os.path.exists(os.path.join(dipath,os.path.split(infile)[1])):
                    print("  Deleting:", os.path.join(dipath,os.path.split(infile)[1]))
                    os.remove(os.path.join(dipath,os.path.split(infile)[1]))
                print("  Retrieving from webdir: ", infile)
                shutil.copy(infile,dipath)
                # Setting permission to defaultuser even if started the job
                uid = pwd.getpwnam(defaultuser)[2]
                gid = grp.getgrnam(defaultgroup)[2]
                os.chown(os.path.join(dipath,os.path.split(infile)[1]),uid,gid)
                # Deleting file from web dir
                try:
                    os.remove(os.path.join(webdir,os.path.split(infile)[1]))
                except:
                    print("  !! No permissions to modify webdirectory")
                    pass


    # copy all files from web directory to the analysis folder
    if getremote:
        delete = True
        if keepremote:
            delete = False
        print (" -------------------------------------")
        print (" Getting remote data - deleting downloaded data from remote source set to {}".format(delete)) 
        dipath = os.path.join(archive,stationid,'DI','analyze')
        for pier in pierlist:
            if not os.path.exists(dipath):
                os.makedirs(dipath)
            diid = pier + '_' + stationid + '.txt'
            try:
                port = mpcred.lc(remotecred,'port')
            except:
                port = 21
            ftpget(mpcred.lc(remotecred,'address'),mpcred.lc(remotecred,'user'),mpcred.lc(remotecred,'passwd'),remotepath,os.path.join(archive,stationid,'DI','analyze'),diid,port= port, delete=delete)

    if debug:
        print (" -------------------------------------")
        print(" DI data defined and collected - now starting the analysis for variometer: {}".format(variolist))

    print (" ")

    # -----------------------------------------------------
    # c) analyze all files in the local analysis directory and put successfully analyzed data to raw
    # -----------------------------------------------------
    for pier in pierlist:
        print("######################################################")
        print("Starting analysis for pier ", pier)
        print("######################################################")
        abspath = dipath
        diid = pier + '_' + stationid + '.txt'
        for vario in variolist:
            dataid = variodataidlist[variolist.index(vario)]
            if os.path.exists(os.path.join(archive,stationid,vario,vario+'_'+dataid)):
                variopath = os.path.join(archive,stationid,vario,vario+'_'+dataid,vario+'*')
            else:
                variopath = vario
                if not os.path.exists(variopath):
                    print(" -> No variometerdata found in the specified paths/IDs - using dummy path")
                    variopath = '/tmp/*'
            print(" -> Using Variometerdata at:", variopath)
            for scalar in scalarlist:
                # Define paths for variometer and scalar data
                scalarid = scalardataidlist[scalarlist.index(scalar)]
                if os.path.exists(os.path.join(archive,stationid,scalar,scalar+'_'+scalarid)):
                    scalarpath = os.path.join(archive,stationid,scalar,scalar+'_'+scalarid,scalar+'*')
                else:
                    scalarpath = scalar
                    if not os.path.exists(scalarpath):
                        print(" -> No scalar data found in the specified paths/IDs - using dummy path")
                        scalarpath = '/tmp/*'
                print(" -> Using Scalar data at:", scalarpath)
                # ALPHA and delta needs to be provided with the database

                print (" -------------------------------------")
                print(" Extracting delta and rotation parameters ... should not be necessary as this is be done by absoluteAnalysis provided a database is connected")
                deltaF = 0.0
                alpha = 0.0
                beta = 0.0

                """
                if db:
                    print(" ")
                    alpha =  dbgetfloat(db, 'DATAINFO', vario, 'DataSensorAzimuth')
                    if not isNumber(alpha):
                        alpha = 0.0
                    beta =  dbgetfloat(db, 'DATAINFO', vario, 'DataSensorTilt')
                    if not isNumber(beta):
                        beta = 0.0
                    deltaF =  dbgetfloat(db, 'DATAINFO', scalar, 'DataDeltaF')
                    if not isNumber(deltaF):
                        deltaF = 0.0
                else:
                    # eventually add an input option
                    # load a scalar file from path and get delta F from header
                    try:
                        scal = read(scalarpath,starttime=begin,endtime=begin)
                        try:
                            scal = applyDeltas(db,scal)
                            deltaF = 0.0
                        except:
                            deltaF = scal.header['DataDeltaF']
                    except:
                        deltaF = 0.0
                    try:
                        var = read(variopath,starttime=begin,endtime=begin)
                        try:
                            var = applyDeltas(db,var)
                        except:
                            pass
                        # TODO this is wrong -> but clarify whether a correction is necessary at all 
                        alpha = var.header['DataSensorAzimuth']
                        beta = var.header['DataSensorTilt']
                    except:
                        alpha = 0.0
                        beta = 0.0
                print("using alpha, beta, deltaF:", alpha, beta, deltaF)
                """

                print (" -------------------------------------")
                print(" Extracting azimuth data ... should be contained in DI files, can be provided as option, is contained in PIERS table of DB")
                # Azimuths are usually contained in the DI files
                ## Eventually overriding azimuths in DI files
                if len(azimuthlist) > 0:
                    azimuth = azimuthlist[pierlist.index(pier)]
                    if azimuth == 'False' or azimuth == 'false':
                        azimuth = False
                else:
                    azimuth = False
                if azimuth:
                    print (" -> Overriding (eventual) DI files data with an azimuth of {} deg".format(azimuth))
                else:
                    print (" -> Using azimuth from DI file") 
                if len(abstypelist) > 0:
                    abstype = abstypelist[pierlist.index(pier)]
                    if abstype == 'False' or abstype == 'false':
                        abstype = False
                else:
                    abstype = False
                if abstype:
                    print (" -> Selected type of absolute measurements is {}".format(abstype))
                else:
                    print (" -> Absolute measurement type taken from DI file")
                # TODO ... Get azimuth data from PIERS table
                if db:
                    print (" Checking azimuth in PIERS table of the database ...")
                    val = dbselect(db,'AzimuthDictionary','PIERS','PierID like "{}"'.format(pier))[0]
                    print ("Found ", val)


                print (" -------------------------------------")
                movetoarchive=False
                if createarchive and variolist.index(vario) == len(variolist)-1  and scalarlist.index(scalar) == len(scalarlist)-1:
                    print(" Running analysis - and moving successfully analyzed files to raw directory")
                    movetoarchive=os.path.join(archive,stationid,'DI','raw')
                else:
                    print(" Running analysis - and keeping files in analyze directory")
                absstream = absoluteAnalysis(abspath,variopath,scalarpath,expD=expD,expI=expI, diid=diid,stationid=stationid,abstype=abstype,azimuth=azimuth,pier=pier, alpha=alpha,deltaF=deltaF,starttime=begin,endtime=end,db=db,dbadd=dbadd,compensation=compensation,magrotation=rotation, movetoarchive=movetoarchive,deltaD=0.0000000001,deltaI=0.0000000001)
                print(" -> Done")

		# -----------------------------------------------------
    		# d) write data to a file and sort it, write it again
                #          (workaround to get sorting correctly)
    		# -----------------------------------------------------
                print (" -------------------------------------")
                if absstream and absstream.length()[0] > 0:
                    print(" Writing {} data line(s) ...".format(absstream.length()[0]))
                    absstream.write(os.path.join(archive,stationid,'DI','data'),coverage='all', mode='replace',filenamebegins=identifier+'_'+vario+'_'+scalar+'_'+pier)
                    try:
                        # Reload all data, delete old file and write again to get correct ordering
                        newabsstream = read(os.path.join(archive,stationid,'DI','data',identifier+'_'+vario+'_'+scalar+'_'+pier+'*'))
                        os.remove(os.path.join(archive,stationid,'DI','data',identifier+'_'+vario+'_'+scalar+'_'+pier+'.txt'))# delete file from hd
                        newabsstream.write(os.path.join(archive,stationid,'DI','data'),coverage='all',mode='replace',filenamebegins=identifier+'_'+vario+'_'+scalar+'_'+pier)
                    except:
                        print (" Stream apparently not existing...")
                    print(" -> Done")
                    if addBLVdb:
                        # SensorID necessary....
                        print(" Adding data to the data bank ... ")
                        #newabsstream.header["SensorID"] = vario
                        writeDB(db,absstream,tablename=identifier+'_'+vario+'_'+scalar+'_'+pier)
                        #stream2db(db,newabsstream,mode='force',tablename=identifier+'_'+vario+'_'+scalar+'_'+pier)
                        print(" -> Done")

        	    # -----------------------------------------------------
        	    # f) get flags and apply them to data
        	    # -----------------------------------------------------
                    if db and flagging and addBLVdb:
                        newabsstream = readDB(db,identifier+'_'+vario+'_'+scalar+'_'+pier)
       	                flaglist = db2flaglist(db,identifier+'_'+vario+'_'+scalar+'_'+pier)
                    elif addBLVdb:
                        newabsstream = readDB(db,identifier+'_'+vario+'_'+scalar+'_'+pier)
                        flaglist = []
                    if len(flaglist) > 0:
                        flabsstream = newabsstream.flag(flaglist)
        	        #for i in range(len(flaglist)):
        	        #    flabsstream = newabsstream.flag_stream(flaglist[i][2],flaglist[i][3],flaglist[i][4],flaglist[i][0],flaglist[i][1])
                        flabsstream.write(os.path.join(archive,stationid,'DI','data'),coverage='all',filenamebegins=identifier+'_'+vario+'_'+scalar+'_'+pier)
                        pltabsstream = flabsstream.remove_flagged()

        	    # -----------------------------------------------------
        	    # h) fit baseline and plot
        	    # -----------------------------------------------------
                    try:
                        #pltabsstream = read(os.path.join(archive,stationid,'DI','data',identifier+'_'+vario+'_'+scalar+'_'+pier+'*'))
               	        pltabsstream.trim(starttime=datetime.utcnow()-timedelta(days=380))
        	        # fit baseline using the parameters defined in db (if parameters not available then skip fitting)
        	        #absstream = absstream.fit(['dx','dy','dz'],poly,4)
                        savename = identifier+'_'+vario+'_'+scalar+'_'+pier+ '.png'
                        #absstream = absstream.extract('f',98999,'<')
                        mp.plot(pltabsstream,['dx','dy','dz'],symbollist=['o','o','o'],plottitle=vario+'_'+scalar+'_'+pier,outfile=os.path.join(archive,stationid,'DI','graphs',savename))
        	        #absstream.plot(['dx','dy','dz'],symbollist=['o','o','o'],plottitle=vario+'_'+scalar+'_'+pier,outfile=os.path.join(archive,stationid,'DI','graphs',savename))
                    except:
                        pass

    # -----------------------------------------------------
    # j) move files from analyze folder to web folder
    # -----------------------------------------------------
    # move only if createarchive is selected
    if createarchive:
        print (" -------------------------------------")
        print(" Dont mind the error message - works only if su at cron is running this job")
        filelst = []
        for infile in iglob(os.path.join(archive,stationid,'DI','analyze','*.txt')):
            print("Processing ", infile)
            filelst.append(infile)
            destination = '/var/www/joomla/images/didaten/'
            infilename = os.path.split(infile)
            print(infilename)
            try:
                shutil.copy(infile, destination)
                #perform changes to privs
                if not webuser == '':
                    uid = pwd.getpwnam(webuser)[2]
                    gid = grp.getgrnam(webgroup)[2]
                    os.chown(os.path.join(destination,infilename[1]),uid,gid)
            except:
                print("Webdir not accessible - finishing")
                pass


    print ("----------------------------------------------------------------")
    print ("di.py app finished")
    print ("----------------------------------------------------------------")
    print ("SUCCESS")

if __name__ == "__main__":
   main(sys.argv[1:])
